/*
 * stepfet/dsoinit.c
 *
 * (C) Copyright J. Zastrau, 2004
 * Released under GPL V2
 * 
 * microstepping controller
 * for MSP1232 devices
 *
 * bring up DSO at specifig frequency
 * PLL using WDT-Interrupt in interval mode
 */

#include "io.h"
#include "config.h"
#include "defs.h"
#include "dcoinit.h"

int
wdt_poll(u16 t_settle)
{
	u16 cnt;					// TimerA capture
	
	NOP_WAIT(t_settle);				// stabilize clock

	WDTCTL = WDTCTL_INIT;				// reset WDT;
	TACTL  = TACLR+TASSEL1+MC1+ID0;			// SMCLK, continuous mode, Prescaler 2 to avoid overflow
	cnt = (TAR_TICKS>>1);				// calculate next capture
	WDT_CLRIFG;					// clear Flag

	while (!(IFG1&WDTIFG));				// poll WDT Flag
	
	u16 mytar = TAR;				// capture Timer A value
	return (cnt-mytar);
}


void dco_init (void)
{
	u16 retry=LFXT_RETRY;				// retry counter
	
	// wait for LFXT1CLK using Timer A
	TACTL = TASSEL0+MC1+TACLR;			// start timer A using ACLK, clear Interrupt flag
	
	do { retry--;					// ACLK works if timer starts counting; loop ~10cycles
	} while ((TAR<1024)&&retry);			// wait some clock cycles or retry 
	
	if (!retry)					// timer A not working -> die with postcode 4;
		die(PNOXT);

	// since we have now a stable ACLK change
	// the WDT timer with 15,625ms period in
	// interval mode and use timer A in continous
	// mode to adjust DCO frequency. 
	DCOCTL = 0x00;
	BCSCTL1 = BCSCTL1_INIT;				// choose high frequency range
	WDTCTL = WDTPW|WDTCNTCL|WDTHOLD;		// clear and stop watchdog before interval change
	
	signed int adjust;
	do {	adjust = wdt_poll(DCO_SETTLE);		// measure period
		if (adjust<0)				// timer A too fast
			BCSCTL1 -=1;			// decrease RSEL value
	} while (adjust<0);				// until DCO frequency too low

	// successive approximation for DCOCTL
	int i;
	for (i=15; i>1;i--) {				// the last to MOD Bits will be handled by IRQ 
		adjust = wdt_poll(DCO_SETTLE>>1);
		DCOCTL |= (1<<i);			// try setting a higher frequency
		if (adjust<0) {				// too fast?
			DCOCTL &= ~(1<<(i+1));		// undo last frequency change
		}
	}

	// DCO is in the right frequency range now, we can switch to PLL mode
	// PLL can lock on multiples of (1/TAR_PERIOD). Restart all timers
	TACCR0 	= TAR_PERIOD-1;				// set period
	TACTL 	= TASSEL1|MC0|TACLR;			// up mode using SMCLK
	TAR	= TAR_ZERO-1;				// Timer A is in phase initially
	WDTCTL = WDTCTL_INIT;				// reset watchdog 
	WDT_CLRIFG;					// enable wdt interrupt, clear Flag first
	IE1     |=  ~WDTIE;
}


