#include "config.h"

/*
 * stepfet/pll.s
 *
 * (C) Copyright J. Zastrau, 2004
 * Watchdog interrupt handler for DCO-PLL
 * inspired by App.-Report SLAA074 (TI, Anton Muehlhofer)
 */

.text

interrupt(WDT_VECTOR)               	; interrupt handler to synchronize DCO
.global dco
dco:	push	r12
	
	mov.b	&DCOCTL,r12		; check limits for DCO
	tst.b	r12			; already lower bound?
	jz	res_dec			; yes - choose different R 
	inv.b	r12			; upper bound (0xff) reached?, r12 clobbered
	jz	res_inc			; yes - choose different R

	mov	&TAR,r12		; capture Timer A
	cmp	#(TAR_ZERO-1),r12	; MODx adjustment, check 1/2 TMRA period 
	jlo	dco_inc			; DCO too slow (TMRA < 50)
	jeq	dco_end			; do nothing if we are in sync

dco_dec:				; TMRA too fast 
	dec.b	&DCOCTL			; decrement DCOCTL / don't use double dec!
	pop	r12
	reti
dco_inc:
	inc.b	&DCOCTL			; increment DCOCTL 
	pop	r12
	reti

res_inc:
	mov.b	&BCSCTL1,r12
	bic.b	#0xf8,r12		; mask BIT0...2
	cmp.b	#7,r12			; already maximum R?
	jge	dco_end			; give up
	inc.b	&BCSCTL1		; choose higher frequency
	mov.b	#0x01,&DCOCTL		; set new DCO value
	pop	r12
	reti

res_dec:
	mov.b	&BCSCTL1,r12
	and.b	#7,r12
	jz	dco_end			; already lowest setting, give up
	dec.b	&BCSCTL1		; lower frequency
	mov.b	#0xfe,&DCOCTL		; set new DCO tap

dco_end:
	pop	r12
        reti

